--This script will run on all levels when mod is active.
--Modding documentation: http://teardowngame.com/modding
--API reference: http://teardowngame.com/modding/api.html

function setKey(key, default, boolean, infinite)

	boolean = boolean or false
	infinite = infinite or false
	
	if not HasKey(key) then
		if boolean then
			SetBool(key, default)
		else
			SetInt(key, default)
		end
	end
	if infinite then
		if not HasKey(key .. "_infinite") then
			SetBool(key .. "_infinite", false)
		end
	end
end

function KeyInit()
	setKey("savegame.mod.fire", true, true)
	setKey("savegame.mod.dca", true, true)
	setKey("savegame.mod.damage", true, true)
	setKey("savegame.mod.machinegun", true, true)
	setKey("savegame.mod.aircraft", true, true)
	setKey("savegame.mod.mortar", true, true)
	setKey("savegame.mod.music", true, true)
	setKey("savegame.mod.lowconfig", false, true)
	setKey("savegame.mod.start", false, true)
	setKey("savegame.mod.flak", true, true)
end

function init()
	
	KeyInit()
	
	FIRE = GetBool("savegame.mod.fire")
	DCA = GetBool("savegame.mod.dca")
	DAMAGE_MAP = GetBool("savegame.mod.damage")
	MACHINEGUN_SND = GetBool("savegame.mod.machinegun")
	AIRCRAFT = GetBool("savegame.mod.aircraft")
	MORTAR = GetBool("savegame.mod.mortar")
	MUSIC = GetBool("savegame.mod.music")
	LOWCONFIG = GetBool("savegame.mod.lowconfig")
	START = not GetBool("savegame.mod.start")
	FLAK = GetBool("savegame.mod.flak")
	
	world = {}
	world.aa, world.bb = GetBodyBounds(GetWorldBody())
	
	if FIRE then
		fire = {}
		fire.spots = {}
		fire.spots = initSpots(10000)
		fire.first = true
	end
	
	if DAMAGE_MAP then
		explosion = {}
		explosion.spots = {}
		explosion.amount = 500
		explosion.defaultIntervalCooldown = 1.0
		explosion.step = 1
		explosion.intervalCooldown = explosion.defaultIntervalCooldown
		explosion.stepCount = 10
		exploPerStep = explosion.amount / explosion.stepCount
		
	end
	
	if DCA then
		dca = {}
		dca.defaultIntervalCooldown = 0.1
		dca.angleRotationPerSec = 5 -- per sec
		dca.spread = 0.03
		dca.defaultCooldown = 5
		dca.defaultRandomSalvo = 10
		dca.defaultSalvo = 10
		dca.spots = {}
		dca.spots = initDCASpots(10)
	end
	
	if MACHINEGUN_SND then
		mg = {}
		mg.spots = {}
		mg.snd = {}
		mg.snd[#mg.snd + 1] = LoadSound("snd/dca.ogg")
		mg.snd[#mg.snd + 1] = LoadSound("snd/gun.ogg")
		mg.snd[#mg.snd + 1] = LoadSound("snd/assault_rifle.ogg")
		mg.count = 3
		mg.defaultIntervalCooldown = 0.1
		mg.defaultCooldown = 3
		mg.defaultRandomSalvo = 4
		mg.defaultSalvo = 1
		mg.soundDist = 20
		mg.spots = initMachinegunSound(mg.count)
	end
	
	if AIRCRAFT then
		ac = {}
		ac.height = 50 + world.bb[2]
		ac.cooldown = 3
		ac.defaultCooldown = 10
		ac.speed = 250
		ac.body = 0
		ac.snd = LoadSound("snd/flyby_a10.ogg")
		ac.transform = Transform(Vec(0, -100, 0))
		local entities = Spawn("MOD/prefabs/airstrike_a10.xml", ac.transform)
		for i=1, #entities do
			if GetEntityType(entities[i]) == "body" then
				ac.body = entities[i]
				break
			end
		end
	end
	
	if MORTAR then
		mortar = {}
		mortar.delay = {
			default = 0.5,
			value = 0.5
		}
		mortar.first = false
		mortar.snd = LoadSound("snd/mortar.ogg")
		mortar.pos = Vec(0, 0, 0)
		mortar.cooldown = {
			default = 1,
			value = 0
		}
		mortar.explosion = 2.0
	end
	
	if MUSIC then
		music = {}
		music.loop = LoadLoop("snd/music.ogg")
	end
	
	if FLAK then
		flak = {
			height = world.bb[2] + 200,
			explosion = 2.0,
			cooldown = {
				default = 0.8,
				value = 0.0
			}
		}
	end
	
end

function tick(dt)
	if START then
		if DCA then
			DCAShoot(dca.spots, dt)
		end
		
		if MACHINEGUN_SND then
			MGShoot(mg.spots, dt)
		end
		
		if AIRCRAFT then
			updateAircraft(dt)
		end
		
		if MORTAR then
			if updateCooldown(mortar.cooldown, 2) then
				mortar.pos = getMortarPos()
				mortar.first = true
				PlaySound(mortar.snd, mortar.pos, 1)
			end
			if updateCooldown(mortar.delay, 0, mortar.first) then
				Explosion(mortar.pos, mortar.explosion)
				mortar.first = false
			end
		end
	else
		if InputPressed('c') then
			START = true
		end
	end
end


function update(dt)
	
	if START then
		if FIRE and fire.first then
			ambiantFire(fire.spots)
			fire.first = false
		end
	
		if DAMAGE_MAP and explosion.step < explosion.stepCount then
			explosion.intervalCooldown = explosion.intervalCooldown - dt
			if explosion.intervalCooldown <= 0 then
				explosion.intervalCooldown = explosion.defaultIntervalCooldown
				explosion.step = explosion.step + 1
				explosion.spots = initSpots(exploPerStep)
				if LOWCONFIG then
					holes(explosion.spots)
				else
					explosions(explosion.spots)
				end
			end
			SetPlayerHealth(1.0)
			clearDebris(2000)
		end
		
		if MUSIC then
			PlayLoop(music.loop, GetPlayerCameraTransform().pos, 0.8)
		end
		
		if FLAK then
			updateFlak(flak, dt)
		end
	end
end


function draw(dt)
	
end

--------------------------------------------------------------------------------

function deepcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

function rand(minv, maxv)
	minv = minv or nil
	maxv = maxv or nil
	if minv == nil then
		return math.random()
	end
	if maxv == nil then
		return math.random(minv)
	end
	return math.random(minv, maxv)
end

function updateCooldown(cd, variation, additionalCondition)
	variation = variation or 0
	if additionalCondition == nil then
		additionalCondition = true
	end
	cd.value = cd.value - GetTimeStep()
	if cd.value <= 0 then
		if variation >= 1 then
			variation = rand(variation)
		end
		cd.value = cd.default + variation
		return additionalCondition
	end
	return false
end

function debugWatchTable(t, addonName)
	addonName = addonName or ""
	for k, v in pairs(t) do
		DebugWatch(addonName .. " " .. k, v)
	end
end

--------------------------------------------------------------------------------

function clearDebris(size)
	size = size or 500
	local GetAllShapes = FindShapes(nil, true)
	for i, shape in ipairs(GetAllShapes) do
		if GetShapeVoxelCount(shape) <= size  and  IsShapeBroken(shape) == true then
			Delete(shape) 
		end
	end
end

function initSpots(count)
	local x = 0
	local y = 0
	local z = 0
	
	local down = Vec(0, -1, 0)
	local maxDist = world.bb[2] - world.aa[2]
	
	local spots = {}
	
	for i=1, count do
		x = rand(world.aa[1], world.bb[1])
		y = world.bb[2]
		z = rand(world.aa[3], world.bb[3])
		local hit, dist, normal, shape = QueryRaycast(Vec(x, y, z), down, maxDist, 0, true)
		if hit then
			local pos = Vec(x, y - dist, z)
			if not IsPointInWater(VecAdd(pos, Vec(0, 0.1, 0))) then
				spots[#spots + 1] = {}
				spots[#spots].pos = deepcopy(pos)
			end
		end
	end
	
	return deepcopy(spots)
end

function getMortarPos()
	local down = Vec(0, -1, 0)
	local maxDist = world.bb[2] - world.aa[2]
	local pos = Vec(0, 0, 0)
	x = rand(world.aa[1], world.bb[1])
	y = world.bb[2]
	z = rand(world.aa[3], world.bb[3])
	local hit, dist, normal, shape = QueryRaycast(Vec(x, y, z), down, maxDist)
	if hit then
		pos = Vec(x, y - dist, z)
	end
	
	return deepcopy(pos)
end

function updateFlak(f, dt)
	if updateCooldown(f.cooldown) then
		local pos = randomPos(world.aa, world.bb)
		pos[2] = flak.height + rand(30) - 15
		Explosion(pos, flak.explosion)
	end
end

function initDCASpots(count)
	local spots = {}
	
	for i=1, count do
		spots[#spots + 1] = {}
		local pos = randomPosBorder()
		--local orientation = GetQuatEuler(QuatLookAt(pos, Vec(randomInterpolate(world.aa[1], world.bb[1]), 0, randomInterpolate(world.aa[3], world.bb[3]))))
		local orientation = rand() * 360 - 180
		local angle = 45
		spots[#spots].t = Transform(deepcopy(pos), QuatEuler(angle, orientation, 0))
		
		spots[#spots].defaultCooldown = dca.defaultCooldown
		spots[#spots].cooldown = rand(spots[#spots].defaultCooldown)
		
		spots[#spots].defaultIntervalCooldown = dca.defaultIntervalCooldown
		spots[#spots].intervalCooldown = 0
		spots[#spots].defaultSalvo = dca.defaultSalvo
		spots[#spots].randomSalvo = dca.defaultRandomSalvo
		spots[#spots].salvo = 0
	end
	
	return deepcopy(spots)
end

function initMachinegunSound(count)
	local spots = {}
	
	local playerPos = GetPlayerCameraTransform().pos
	
	for i=1, count do
		spots[#spots + 1] = {}
		local pos = VecAdd(playerPos, Vec(rand(mg.soundDist) - mg.soundDist / 2, 0, rand(mg.soundDist) - mg.soundDist / 2))
		
		spots[#spots].pos = pos
		
		spots[#spots].snd = 0
		
		spots[#spots].defaultCooldown = mg.defaultCooldown
		spots[#spots].cooldown = rand(spots[#spots].defaultCooldown)
		
		spots[#spots].defaultIntervalCooldown = mg.defaultIntervalCooldown
		spots[#spots].intervalCooldown = 0
		spots[#spots].defaultSalvo = mg.defaultSalvo
		spots[#spots].randomSalvo = mg.defaultRandomSalvo
		spots[#spots].salvo = 0
	end
	
	return deepcopy(spots)
end

function randomInterpolate(a, b)
	return math.random() * (b - a) + a
end

function randomPos(aa, bb)
	return Vec(randomInterpolate(aa[1], bb[1]), randomInterpolate(aa[2], bb[2]), randomInterpolate(aa[3], bb[3]))
end

function randomPosBorder()
	local p = Vec(0, 0, 0)
	p[1] = randomInterpolate(world.aa[1], world.bb[1])
	p[3] = randomInterpolate(world.aa[3], world.bb[3])
	local rd = rand(4)
	if rd == 1 then
		p[1] = world.aa[1]
	elseif rd == 2 then
		p[3] = world.aa[3]
	elseif rd == 3 then
		p[1] = world.bb[1]
	else
		p[3] = world.bb[3]
	end
	return deepcopy(p)
end

function ambiantFire(spots)
	for i=1, #spots do
		SpawnFire(spots[i].pos)
	end
end

function explosions(spots)
	for i=1, #spots do
		Explosion(spots[i].pos, 2.0)
	end
end

function holes(spots)
	for i=1, #spots do
		MakeHole(spots[i].pos, 4.0, 3.0, 2.0, true)
	end
end

function DCAShoot(spots, dt)
	for i=1, #spots do
		spots[i].cooldown = spots[i].cooldown - dt
		--DrawLine(spots[i].t.pos, TransformToParentPoint(spots[i].t, VecScale(Vec(0, 0, -1), 600)), 1, 0, 0, 0.5)
		local forward = Vec(rand() * dca.spread - dca.spread / 2, rand() * dca.spread - dca.spread / 2, -1)
		
		if spots[i].cooldown <= 0 and spots[i].salvo <= 0 then
		
			spots[i].salvo = rand(spots[i].randomSalvo) + spots[i].defaultSalvo
			
		elseif spots[i].salvo > 0 then
		
			if spots[i].intervalCooldown <= 0 then
			
				spots[i].intervalCooldown = spots[i].defaultIntervalCooldown
				spots[i].salvo = spots[i].salvo - 1
				Shoot(spots[i].t.pos, QuatRotateVec(spots[i].t.rot, forward), "bullet", 1, 600)
				--PlaySound(DCASound, spots[i].t.pos, 1)
				if spots[i].salvo == 0 then
					spots[i].cooldown = spots[i].defaultCooldown + rand(2)
				end
				
			else
			
				spots[i].intervalCooldown = spots[i].intervalCooldown - dt
				
			end
			
		end
	end
end

function MGShoot(spots, dt)

	local playerPos = GetPlayerCameraTransform().pos
	--DebugPrint(#spots)
	for i=1, #spots do
		--DebugCross(spots[i].pos)
		--DebugLine(spots[i].pos, playerPos)
		spots[i].cooldown = spots[i].cooldown - dt
		
		if spots[i].cooldown <= 0 and spots[i].salvo <= 0 then
			
			spots[i].salvo = rand(spots[i].randomSalvo) + spots[i].defaultSalvo
			local offset = Vec(rand(mg.soundDist) - mg.soundDist / 2, 0, rand(mg.soundDist) - mg.soundDist / 2)
			spots[i].pos = VecAdd(playerPos, VecScale(offset, mg.soundDist / VecLength(offset)))
			spots[i].snd = mg.snd[rand(#mg.snd)]
			
		elseif spots[i].salvo > 0 then
		
			if spots[i].intervalCooldown <= 0 then
			
				spots[i].intervalCooldown = spots[i].defaultIntervalCooldown
				spots[i].salvo = spots[i].salvo - 1
				PlaySound(spots[i].snd, spots[i].pos, 1)
				if spots[i].salvo == 0 then
					spots[i].cooldown = 1 + rand(spots[i].defaultCooldown)
				end
				
			else
			
				spots[i].intervalCooldown = spots[i].intervalCooldown - dt
				
			end
			
		end
	end
end

function updateAircraft(dt)
	ac.cooldown = ac.cooldown - dt
	
	if ac.cooldown <= 0 then
		ac.cooldown = ac.defaultCooldown + rand(5)
		local pos = randomPosBorder()
		local playerPos = GetPlayerCameraTransform().pos
		local dir = QuatLookAt(pos, playerPos)
		local velocity = Vec(0, ac.height, -ac.speed)
		velocity = QuatRotateVec(dir, velocity)
		ac.transform = Transform(pos, dir)
		
		PlaySound(ac.snd, playerPos, 1)
		SetBodyTransform(ac.body, ac.transform)
		SetBodyVelocity(ac.body, velocity)
	end
	
end

















































